# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.uuid.ttypes
import concrete.language.ttypes
import concrete.structure.ttypes
import concrete.entities.ttypes
import concrete.situations.ttypes
import concrete.audio.ttypes
import concrete.metadata.ttypes
import concrete.clustering.ttypes

from thrift.transport import TTransport
all_structs = []


class CommunicationTagging(object):
    """
    A structure that represents a 'tagging' of a Communication. These
    might be labels or annotations on a particular communcation.

    For example, this structure might be used to describe the topics
    discussed in a Communication. The taggingType might be 'topic', and
    the tagList might include 'politics' and 'science'.

    Attributes:
     - uuid: A unique identifier for this CommunicationTagging object.
     - metadata: AnnotationMetadata to support this CommunicationTagging object.
     - taggingType: A string that captures the type of this CommunicationTagging
    object. For example: 'topic' or 'gender'.
     - tagList: A list of strings that represent different tags related to the taggingType.
    For example, if the taggingType is 'topic', some example tags might be
    'politics', 'science', etc.
     - confidenceList: A list of doubles, parallel to the list of strings in tagList,
    that indicate the confidences of each tag.

    """


    def __init__(self, uuid=None, metadata=None, taggingType=None, tagList=None, confidenceList=None,):
        self.uuid = uuid
        self.metadata = metadata
        self.taggingType = taggingType
        self.tagList = tagList
        self.confidenceList = confidenceList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.taggingType = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.tagList = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        self.tagList.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.confidenceList = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = iprot.readDouble()
                        self.confidenceList.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('CommunicationTagging')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 2)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.taggingType is not None:
            oprot.writeFieldBegin('taggingType', TType.STRING, 3)
            oprot.writeString(self.taggingType.encode('utf-8') if sys.version_info[0] == 2 else self.taggingType)
            oprot.writeFieldEnd()
        if self.tagList is not None:
            oprot.writeFieldBegin('tagList', TType.LIST, 4)
            oprot.writeListBegin(TType.STRING, len(self.tagList))
            for iter12 in self.tagList:
                oprot.writeString(iter12.encode('utf-8') if sys.version_info[0] == 2 else iter12)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.confidenceList is not None:
            oprot.writeFieldBegin('confidenceList', TType.LIST, 5)
            oprot.writeListBegin(TType.DOUBLE, len(self.confidenceList))
            for iter13 in self.confidenceList:
                oprot.writeDouble(iter13)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.metadata is None:
            raise TProtocolException(message='Required field metadata is unset!')
        if self.taggingType is None:
            raise TProtocolException(message='Required field taggingType is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Communication(object):
    """
    A single communication instance, containing linguistic content
    generated by a single speaker or author.  This type is used for
    both inter-personal communications (such as phone calls or
    conversations) and third-party communications (such as news
    articles).

    Each communication instance is grounded by its original
    (unannotated) contents, which should be stored in either the
    "text" field (for text communications) or the "audio" field (for
    audio communications).  If the communication is not available in
    its original form, then these fields should store the
    communication in the least-processed form available.

    Attributes:
     - id: Stable identifier for this communication, identifying both the
    name of the source corpus and the document that it corresponds to
    in that corpus.
     - uuid: Universally unique identifier for this communication instance.
    This is generated randomly, and can *not* be mapped back to the
    source corpus. It is used as a target for symbolic "pointers".
     - type: A short, corpus-specific term characterizing the nature of the
    communication; may change in a future version of concrete.
    Often used for filtering.  For example, Gigaword uses
    the type "story" to distinguish typical news articles from
    weekly summaries ("multi"), editorial advisories ("advis"), etc.
    At present, this value is typically a literal form from the
    originating corpus: as a result, a type marked 'other' may have
    different meanings across different corpora.
     - text: The full text contents of this communication in its original
    form, or in the least-processed form available, if the original
    is not available.
     - startTime: The time when this communication started (in unix time UTC --
    i.e., seconds since January 1, 1970).
     - endTime: The time when this communication ended (in unix time UTC --
    i.e., seconds since January 1, 1970).
     - communicationTaggingList: A list of CommunicationTagging objects that can support this
    Communication. CommunicationTagging objects can be used to
    annotate Communications with topics, gender identification, etc.
     - metadata: metadata.AnnotationMetadata to support this particular communication.

    Communications derived from other communications should
    indicate in this metadata object their dependency
    to the original communication ID.
     - keyValueMap: A catch-all store of keys and values. Use sparingly!
     - lidList: Theories about the languages that are present in this
    communication.
     - sectionList: Theory about the block structure of this communication.
     - entityMentionSetList: Theories about which spans of text are used to mention entities
    in this communication.
     - entitySetList: Theories about what entities are discussed in this
    communication, with pointers to individual mentions.
     - situationMentionSetList: Theories about what situations are explicitly mentioned in this
    communication.
     - situationSetList: Theories about what situations are asserted in this
    communication.
     - originalText: Optional original text field that points back to an original
    communication.

    This field can be populated for sake of convenience when creating
    "perspective" communication (communications that are based on
    highly destructive changes to an original communication [e.g.,
    via MT]). This allows developers to quickly access the original
    text that this perspective communication is based off of.
     - sound: The full audio contents of this communication in its original
    form, or in the least-processed form available, if the original
    is not available.
     - communicationMetadata: Metadata about this specific Communication, such as information
    about its author, information specific to this Communication
    or Communications like it (info from an API, for example), etc.

    """


    def __init__(self, id=None, uuid=None, type=None, text=None, startTime=None, endTime=None, communicationTaggingList=None, metadata=None, keyValueMap=None, lidList=None, sectionList=None, entityMentionSetList=None, entitySetList=None, situationMentionSetList=None, situationSetList=None, originalText=None, sound=None, communicationMetadata=None,):
        self.id = id
        self.uuid = uuid
        self.type = type
        self.text = text
        self.startTime = startTime
        self.endTime = endTime
        self.communicationTaggingList = communicationTaggingList
        self.metadata = metadata
        self.keyValueMap = keyValueMap
        self.lidList = lidList
        self.sectionList = sectionList
        self.entityMentionSetList = entityMentionSetList
        self.entitySetList = entitySetList
        self.situationMentionSetList = situationMentionSetList
        self.situationSetList = situationSetList
        self.originalText = originalText
        self.sound = sound
        self.communicationMetadata = communicationMetadata

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.id = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.type = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.text = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.I64:
                    self.startTime = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.I64:
                    self.endTime = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.LIST:
                    self.communicationTaggingList = []
                    (_etype17, _size14) = iprot.readListBegin()
                    for _i18 in range(_size14):
                        _elem19 = CommunicationTagging()
                        _elem19.read(iprot)
                        self.communicationTaggingList.append(_elem19)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.MAP:
                    self.keyValueMap = {}
                    (_ktype21, _vtype22, _size20) = iprot.readMapBegin()
                    for _i24 in range(_size20):
                        _key25 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        _val26 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        self.keyValueMap[_key25] = _val26
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 10:
                if ftype == TType.LIST:
                    self.lidList = []
                    (_etype30, _size27) = iprot.readListBegin()
                    for _i31 in range(_size27):
                        _elem32 = concrete.language.ttypes.LanguageIdentification()
                        _elem32.read(iprot)
                        self.lidList.append(_elem32)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 11:
                if ftype == TType.LIST:
                    self.sectionList = []
                    (_etype36, _size33) = iprot.readListBegin()
                    for _i37 in range(_size33):
                        _elem38 = concrete.structure.ttypes.Section()
                        _elem38.read(iprot)
                        self.sectionList.append(_elem38)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 12:
                if ftype == TType.LIST:
                    self.entityMentionSetList = []
                    (_etype42, _size39) = iprot.readListBegin()
                    for _i43 in range(_size39):
                        _elem44 = concrete.entities.ttypes.EntityMentionSet()
                        _elem44.read(iprot)
                        self.entityMentionSetList.append(_elem44)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 13:
                if ftype == TType.LIST:
                    self.entitySetList = []
                    (_etype48, _size45) = iprot.readListBegin()
                    for _i49 in range(_size45):
                        _elem50 = concrete.entities.ttypes.EntitySet()
                        _elem50.read(iprot)
                        self.entitySetList.append(_elem50)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 14:
                if ftype == TType.LIST:
                    self.situationMentionSetList = []
                    (_etype54, _size51) = iprot.readListBegin()
                    for _i55 in range(_size51):
                        _elem56 = concrete.situations.ttypes.SituationMentionSet()
                        _elem56.read(iprot)
                        self.situationMentionSetList.append(_elem56)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 15:
                if ftype == TType.LIST:
                    self.situationSetList = []
                    (_etype60, _size57) = iprot.readListBegin()
                    for _i61 in range(_size57):
                        _elem62 = concrete.situations.ttypes.SituationSet()
                        _elem62.read(iprot)
                        self.situationSetList.append(_elem62)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 16:
                if ftype == TType.STRING:
                    self.originalText = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 20:
                if ftype == TType.STRUCT:
                    self.sound = concrete.audio.ttypes.Sound()
                    self.sound.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 21:
                if ftype == TType.STRUCT:
                    self.communicationMetadata = concrete.metadata.ttypes.CommunicationMetadata()
                    self.communicationMetadata.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Communication')
        if self.id is not None:
            oprot.writeFieldBegin('id', TType.STRING, 1)
            oprot.writeString(self.id.encode('utf-8') if sys.version_info[0] == 2 else self.id)
            oprot.writeFieldEnd()
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 2)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.STRING, 3)
            oprot.writeString(self.type.encode('utf-8') if sys.version_info[0] == 2 else self.type)
            oprot.writeFieldEnd()
        if self.text is not None:
            oprot.writeFieldBegin('text', TType.STRING, 4)
            oprot.writeString(self.text.encode('utf-8') if sys.version_info[0] == 2 else self.text)
            oprot.writeFieldEnd()
        if self.startTime is not None:
            oprot.writeFieldBegin('startTime', TType.I64, 5)
            oprot.writeI64(self.startTime)
            oprot.writeFieldEnd()
        if self.endTime is not None:
            oprot.writeFieldBegin('endTime', TType.I64, 6)
            oprot.writeI64(self.endTime)
            oprot.writeFieldEnd()
        if self.communicationTaggingList is not None:
            oprot.writeFieldBegin('communicationTaggingList', TType.LIST, 7)
            oprot.writeListBegin(TType.STRUCT, len(self.communicationTaggingList))
            for iter63 in self.communicationTaggingList:
                iter63.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 8)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.keyValueMap is not None:
            oprot.writeFieldBegin('keyValueMap', TType.MAP, 9)
            oprot.writeMapBegin(TType.STRING, TType.STRING, len(self.keyValueMap))
            for kiter64, viter65 in self.keyValueMap.items():
                oprot.writeString(kiter64.encode('utf-8') if sys.version_info[0] == 2 else kiter64)
                oprot.writeString(viter65.encode('utf-8') if sys.version_info[0] == 2 else viter65)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        if self.lidList is not None:
            oprot.writeFieldBegin('lidList', TType.LIST, 10)
            oprot.writeListBegin(TType.STRUCT, len(self.lidList))
            for iter66 in self.lidList:
                iter66.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.sectionList is not None:
            oprot.writeFieldBegin('sectionList', TType.LIST, 11)
            oprot.writeListBegin(TType.STRUCT, len(self.sectionList))
            for iter67 in self.sectionList:
                iter67.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.entityMentionSetList is not None:
            oprot.writeFieldBegin('entityMentionSetList', TType.LIST, 12)
            oprot.writeListBegin(TType.STRUCT, len(self.entityMentionSetList))
            for iter68 in self.entityMentionSetList:
                iter68.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.entitySetList is not None:
            oprot.writeFieldBegin('entitySetList', TType.LIST, 13)
            oprot.writeListBegin(TType.STRUCT, len(self.entitySetList))
            for iter69 in self.entitySetList:
                iter69.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.situationMentionSetList is not None:
            oprot.writeFieldBegin('situationMentionSetList', TType.LIST, 14)
            oprot.writeListBegin(TType.STRUCT, len(self.situationMentionSetList))
            for iter70 in self.situationMentionSetList:
                iter70.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.situationSetList is not None:
            oprot.writeFieldBegin('situationSetList', TType.LIST, 15)
            oprot.writeListBegin(TType.STRUCT, len(self.situationSetList))
            for iter71 in self.situationSetList:
                iter71.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.originalText is not None:
            oprot.writeFieldBegin('originalText', TType.STRING, 16)
            oprot.writeString(self.originalText.encode('utf-8') if sys.version_info[0] == 2 else self.originalText)
            oprot.writeFieldEnd()
        if self.sound is not None:
            oprot.writeFieldBegin('sound', TType.STRUCT, 20)
            self.sound.write(oprot)
            oprot.writeFieldEnd()
        if self.communicationMetadata is not None:
            oprot.writeFieldBegin('communicationMetadata', TType.STRUCT, 21)
            self.communicationMetadata.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.id is None:
            raise TProtocolException(message='Required field id is unset!')
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.type is None:
            raise TProtocolException(message='Required field type is unset!')
        if self.metadata is None:
            raise TProtocolException(message='Required field metadata is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class CommunicationSet(object):
    """
    A structure that represents a collection of Communications.

    Attributes:
     - communicationIdList: A list of Communication UUIDs that this CommunicationSet
    represents.

    This field may be absent if this CommunicationSet represents
    a large corpus. If absent, 'corpus' field should be present.
     - corpus: The name of a corpus or other document body that this
    CommunicationSet represents.

    Should be present if 'communicationIdList' is absent.
     - entityMentionClusterList: A list of Clustering objects that represent a
    group of EntityMentions that are a part of this
    CommunicationSet.
     - entityClusterList: A list of Clustering objects that represent a
    group of Entities that are a part of this
    CommunicationSet.
     - situationMentionClusterList: A list of Clustering objects that represent a
    group of SituationMentions that are a part of this
    CommunicationSet.
     - situationClusterList: A list of Clustering objects that represent a
    group of Situations that are a part of this
    CommunicationSet.

    """


    def __init__(self, communicationIdList=None, corpus=None, entityMentionClusterList=None, entityClusterList=None, situationMentionClusterList=None, situationClusterList=None,):
        self.communicationIdList = communicationIdList
        self.corpus = corpus
        self.entityMentionClusterList = entityMentionClusterList
        self.entityClusterList = entityClusterList
        self.situationMentionClusterList = situationMentionClusterList
        self.situationClusterList = situationClusterList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.communicationIdList = []
                    (_etype75, _size72) = iprot.readListBegin()
                    for _i76 in range(_size72):
                        _elem77 = concrete.uuid.ttypes.UUID()
                        _elem77.read(iprot)
                        self.communicationIdList.append(_elem77)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.corpus = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.entityMentionClusterList = []
                    (_etype81, _size78) = iprot.readListBegin()
                    for _i82 in range(_size78):
                        _elem83 = concrete.clustering.ttypes.Clustering()
                        _elem83.read(iprot)
                        self.entityMentionClusterList.append(_elem83)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.entityClusterList = []
                    (_etype87, _size84) = iprot.readListBegin()
                    for _i88 in range(_size84):
                        _elem89 = concrete.clustering.ttypes.Clustering()
                        _elem89.read(iprot)
                        self.entityClusterList.append(_elem89)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.situationMentionClusterList = []
                    (_etype93, _size90) = iprot.readListBegin()
                    for _i94 in range(_size90):
                        _elem95 = concrete.clustering.ttypes.Clustering()
                        _elem95.read(iprot)
                        self.situationMentionClusterList.append(_elem95)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.LIST:
                    self.situationClusterList = []
                    (_etype99, _size96) = iprot.readListBegin()
                    for _i100 in range(_size96):
                        _elem101 = concrete.clustering.ttypes.Clustering()
                        _elem101.read(iprot)
                        self.situationClusterList.append(_elem101)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('CommunicationSet')
        if self.communicationIdList is not None:
            oprot.writeFieldBegin('communicationIdList', TType.LIST, 1)
            oprot.writeListBegin(TType.STRUCT, len(self.communicationIdList))
            for iter102 in self.communicationIdList:
                iter102.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.corpus is not None:
            oprot.writeFieldBegin('corpus', TType.STRING, 2)
            oprot.writeString(self.corpus.encode('utf-8') if sys.version_info[0] == 2 else self.corpus)
            oprot.writeFieldEnd()
        if self.entityMentionClusterList is not None:
            oprot.writeFieldBegin('entityMentionClusterList', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.entityMentionClusterList))
            for iter103 in self.entityMentionClusterList:
                iter103.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.entityClusterList is not None:
            oprot.writeFieldBegin('entityClusterList', TType.LIST, 4)
            oprot.writeListBegin(TType.STRUCT, len(self.entityClusterList))
            for iter104 in self.entityClusterList:
                iter104.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.situationMentionClusterList is not None:
            oprot.writeFieldBegin('situationMentionClusterList', TType.LIST, 5)
            oprot.writeListBegin(TType.STRUCT, len(self.situationMentionClusterList))
            for iter105 in self.situationMentionClusterList:
                iter105.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.situationClusterList is not None:
            oprot.writeFieldBegin('situationClusterList', TType.LIST, 6)
            oprot.writeListBegin(TType.STRUCT, len(self.situationClusterList))
            for iter106 in self.situationClusterList:
                iter106.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(CommunicationTagging)
CommunicationTagging.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 2
    (3, TType.STRING, 'taggingType', 'UTF8', None, ),  # 3
    (4, TType.LIST, 'tagList', (TType.STRING, 'UTF8', False), None, ),  # 4
    (5, TType.LIST, 'confidenceList', (TType.DOUBLE, None, False), None, ),  # 5
)
all_structs.append(Communication)
Communication.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'id', 'UTF8', None, ),  # 1
    (2, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 2
    (3, TType.STRING, 'type', 'UTF8', None, ),  # 3
    (4, TType.STRING, 'text', 'UTF8', None, ),  # 4
    (5, TType.I64, 'startTime', None, None, ),  # 5
    (6, TType.I64, 'endTime', None, None, ),  # 6
    (7, TType.LIST, 'communicationTaggingList', (TType.STRUCT, [CommunicationTagging, None], False), None, ),  # 7
    (8, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 8
    (9, TType.MAP, 'keyValueMap', (TType.STRING, 'UTF8', TType.STRING, 'UTF8', False), None, ),  # 9
    (10, TType.LIST, 'lidList', (TType.STRUCT, [concrete.language.ttypes.LanguageIdentification, None], False), None, ),  # 10
    (11, TType.LIST, 'sectionList', (TType.STRUCT, [concrete.structure.ttypes.Section, None], False), None, ),  # 11
    (12, TType.LIST, 'entityMentionSetList', (TType.STRUCT, [concrete.entities.ttypes.EntityMentionSet, None], False), None, ),  # 12
    (13, TType.LIST, 'entitySetList', (TType.STRUCT, [concrete.entities.ttypes.EntitySet, None], False), None, ),  # 13
    (14, TType.LIST, 'situationMentionSetList', (TType.STRUCT, [concrete.situations.ttypes.SituationMentionSet, None], False), None, ),  # 14
    (15, TType.LIST, 'situationSetList', (TType.STRUCT, [concrete.situations.ttypes.SituationSet, None], False), None, ),  # 15
    (16, TType.STRING, 'originalText', 'UTF8', None, ),  # 16
    None,  # 17
    None,  # 18
    None,  # 19
    (20, TType.STRUCT, 'sound', [concrete.audio.ttypes.Sound, None], None, ),  # 20
    (21, TType.STRUCT, 'communicationMetadata', [concrete.metadata.ttypes.CommunicationMetadata, None], None, ),  # 21
)
all_structs.append(CommunicationSet)
CommunicationSet.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'communicationIdList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 1
    (2, TType.STRING, 'corpus', 'UTF8', None, ),  # 2
    (3, TType.LIST, 'entityMentionClusterList', (TType.STRUCT, [concrete.clustering.ttypes.Clustering, None], False), None, ),  # 3
    (4, TType.LIST, 'entityClusterList', (TType.STRUCT, [concrete.clustering.ttypes.Clustering, None], False), None, ),  # 4
    (5, TType.LIST, 'situationMentionClusterList', (TType.STRUCT, [concrete.clustering.ttypes.Clustering, None], False), None, ),  # 5
    (6, TType.LIST, 'situationClusterList', (TType.STRUCT, [concrete.clustering.ttypes.Clustering, None], False), None, ),  # 6
)
fix_spec(all_structs)
del all_structs



from __future__ import unicode_literals
from concrete.util import (
    RedisHashBackedStoreHandler,
    S3BackedStoreHandler,
    prefix_s3_key,
    unprefix_s3_key,
)
from concrete import ServiceInfo

from mock import Mock, sentinel, patch


def test_redis_hash_backed_store_handler_about():
    redis_db = Mock()
    key = Mock()
    handler = RedisHashBackedStoreHandler(redis_db, key)
    assert isinstance(handler.about(), ServiceInfo)


def test_redis_hash_backed_store_handler_alive():
    redis_db = Mock()
    key = Mock()
    handler = RedisHashBackedStoreHandler(redis_db, key)
    assert handler.alive()


@patch('concrete.util.access.RedisCommunicationWriter')
def test_redis_hash_backed_store_handler_store(mock_redis_communication_writer_class):
    mock_redis_communication_writer = mock_redis_communication_writer_class.return_value
    redis_db = Mock()
    key = Mock()

    comm = Mock()
    comm.id = sentinel.comm_id
    handler = RedisHashBackedStoreHandler(redis_db, key)
    mock_redis_communication_writer_class.assert_called_once_with(redis_db, key, key_type='hash')

    handler.store(comm)
    mock_redis_communication_writer.write.assert_called_once_with(comm)


def test_s3_backed_store_handler_about():
    bucket = Mock()
    handler = S3BackedStoreHandler(bucket)
    assert isinstance(handler.about(), ServiceInfo)


def test_s3_backed_store_handler_alive():
    bucket = Mock()
    handler = S3BackedStoreHandler(bucket)
    assert handler.alive()


def test_prefix_s3_key():
    # MD5 of 'asdf' starts with '912e'
    assert prefix_s3_key('asdf', 0) == 'asdf'
    assert prefix_s3_key('asdf', 1) == '9asdf'
    assert prefix_s3_key('asdf', 2) == '91asdf'


def test_unprefix_s3_key():
    assert unprefix_s3_key('xyasdf', 0) == 'xyasdf'
    assert unprefix_s3_key('xyasdf', 1) == 'yasdf'
    assert unprefix_s3_key('xyasdf', 2) == 'asdf'


@patch('concrete.util.access.prefix_s3_key')
@patch('concrete.util.access.write_communication_to_buffer')
def test_s3_backed_store_handler_store(
        mock_write_communication_to_buffer,
        mock_prefix_s3_key):
    key = Mock()
    bucket = Mock(get_key=Mock(side_effect=[key]))

    comm = Mock()
    comm.id = sentinel.comm_id
    handler = S3BackedStoreHandler(bucket, sentinel.prefix_len)

    mock_prefix_s3_key.side_effect = [sentinel.prefixed_key]
    mock_write_communication_to_buffer.side_effect = [sentinel.buf]
    handler.store(comm)
    bucket.get_key.assert_called_once_with(sentinel.prefixed_key, validate=False)
    mock_prefix_s3_key.assert_called_once_with(sentinel.comm_id, sentinel.prefix_len)
    mock_write_communication_to_buffer.assert_called_once_with(comm)
    key.set_contents_from_string.assert_called_once_with(sentinel.buf)



from setuptools import setup
import glob
import re


VERSION_FILE_PATH = 'concrete/version.py'
VERSION_RE = re.compile(
    r'^__version__ = (?P<quote>[\'"])(?P<version>[0-9]+\.[0-9]+\.[0-9]+(?:\.dev[0-9]+)?)(?P=quote)$'
)


with open("README.rst", "r", encoding="utf-8") as fh:
    long_description = fh.read()


def get_version():
    with open(VERSION_FILE_PATH) as f:
        for line in f:
            m = VERSION_RE.match(line.rstrip())
            if m is not None:
                return m.group('version')
    raise Exception('unable to determine version from %s' % VERSION_FILE_PATH)


if __name__ == '__main__':
    setup(
        name="concrete",
        version=get_version(),
        description="Python modules and scripts for working with Concrete",
        long_description=long_description,

        packages=[
            'concrete',

            # Python code generated by Thrift Compiler
            'concrete.access',
            'concrete.annotate',
            'concrete.audio',
            'concrete.clustering',
            'concrete.communication',
            'concrete.context',
            'concrete.email',
            'concrete.entities',
            'concrete.exceptions',
            'concrete.language',
            'concrete.learn',
            'concrete.linking',
            'concrete.metadata',
            'concrete.nitf',
            'concrete.search',
            'concrete.services',
            'concrete.services.results',
            'concrete.situations',
            'concrete.spans',
            'concrete.structure',
            'concrete.summarization',
            'concrete.twitter',
            'concrete.uuid',

            # Python code generated by people
            'concrete.util',
        ],

        scripts=glob.glob('scripts/*.py')
                  + glob.glob('concrete/services/*-remote'),

        install_requires=[
            'boto',
            'bottle',
            'humanfriendly',
            'networkx<2.8',
            'thrift==0.16.0',
            'redis>=2.10.0',
            'pycountry>=17.9.23',
            'requests',
        ],

        url="https://github.com/hltcoe/concrete-python",
        classifiers=[
            'Development Status :: 4 - Beta',
            #'Development Status :: 5 - Production/Stable',
            'Environment :: Console',
            'Environment :: No Input/Output (Daemon)',
            'Intended Audience :: Developers',
            'Intended Audience :: Education',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: BSD License',
            'Natural Language :: English',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
            'Programming Language :: Python :: 3 :: Only',
            'Topic :: Database :: Front-Ends',
            'Topic :: Multimedia :: Sound/Audio :: Speech',
            'Topic :: Scientific/Engineering :: Artificial Intelligence',
            'Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator',
            'Topic :: Text Processing :: Linguistic',
            'Topic :: Utilities',
        ],
    )



#!/usr/bin/env python

import argparse

from thrift import TSerialization
from thrift.protocol import TJSONProtocol
import unicodecsv

from concrete import Communication
from concrete.util import CommunicationWriterZip


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('csv_file')
    parser.add_argument('comms_zip_file')
    parser.add_argument('--comm-field', default='Answer.modified_comm')
    args = parser.parse_args()

    csv_fh = open(args.csv_file, 'rb')
    reader = unicodecsv.DictReader(csv_fh)

    with CommunicationWriterZip(args.comms_zip_file) as writer:
        for row in reader:
            json_comm = row[args.comm_field]
            comm = Communication()
            TSerialization.deserialize(
                comm, json_comm.encode('utf-8'),
                protocol_factory=TJSONProtocol.TJSONProtocolFactory())
            writer.write(comm, comm.id + '.comm')


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.services.Service
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(concrete.services.Service.Iface):
    """
    A service that provides an alternative to Annotate,
    with the ability to pass along an additional Context
    parameter that conveys additional information about the
    Communication.

    """
    def annotate(self, original, context):
        """
        Takes a Communication and a Context as input
        and returns a new one as output.

        It is up to the implementing service to verify that
        the input communication is valid, as well as interpret
        the Context in an appropriate manner.

        Can throw a ConcreteThriftException upon error
        (invalid input, analytic exception, etc.).

        Parameters:
         - original
         - context

        """
        pass


class Client(concrete.services.Service.Client, Iface):
    """
    A service that provides an alternative to Annotate,
    with the ability to pass along an additional Context
    parameter that conveys additional information about the
    Communication.

    """
    def __init__(self, iprot, oprot=None):
        concrete.services.Service.Client.__init__(self, iprot, oprot)

    def annotate(self, original, context):
        """
        Takes a Communication and a Context as input
        and returns a new one as output.

        It is up to the implementing service to verify that
        the input communication is valid, as well as interpret
        the Context in an appropriate manner.

        Can throw a ConcreteThriftException upon error
        (invalid input, analytic exception, etc.).

        Parameters:
         - original
         - context

        """
        self.send_annotate(original, context)
        return self.recv_annotate()

    def send_annotate(self, original, context):
        self._oprot.writeMessageBegin('annotate', TMessageType.CALL, self._seqid)
        args = annotate_args()
        args.original = original
        args.context = context
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_annotate(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = annotate_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "annotate failed: unknown result")


class Processor(concrete.services.Service.Processor, Iface, TProcessor):
    def __init__(self, handler):
        concrete.services.Service.Processor.__init__(self, handler)
        self._processMap["annotate"] = Processor.process_annotate
        self._on_message_begin = None

    def on_message_begin(self, func):
        self._on_message_begin = func

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if self._on_message_begin:
            self._on_message_begin(name, type, seqid)
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_annotate(self, seqid, iprot, oprot):
        args = annotate_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = annotate_result()
        try:
            result.success = self._handler.annotate(args.original, args.context)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.exceptions.ttypes.ConcreteThriftException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("annotate", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class annotate_args(object):
    """
    Attributes:
     - original
     - context

    """


    def __init__(self, original=None, context=None,):
        self.original = original
        self.context = context

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.original = concrete.communication.ttypes.Communication()
                    self.original.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.context = concrete.context.ttypes.Context()
                    self.context.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('annotate_args')
        if self.original is not None:
            oprot.writeFieldBegin('original', TType.STRUCT, 1)
            self.original.write(oprot)
            oprot.writeFieldEnd()
        if self.context is not None:
            oprot.writeFieldBegin('context', TType.STRUCT, 2)
            self.context.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(annotate_args)
annotate_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'original', [concrete.communication.ttypes.Communication, None], None, ),  # 1
    (2, TType.STRUCT, 'context', [concrete.context.ttypes.Context, None], None, ),  # 2
)


class annotate_result(object):
    """
    Attributes:
     - success
     - ex

    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = concrete.communication.ttypes.Communication()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.exceptions.ttypes.ConcreteThriftException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('annotate_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(annotate_result)
annotate_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [concrete.communication.ttypes.Communication, None], None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.exceptions.ttypes.ConcreteThriftException, None], None, ),  # 1
)
fix_spec(all_structs)
del all_structs



from __future__ import unicode_literals
from .mem_io import (
    read_communication_from_buffer,
    write_communication_to_buffer
)

from uuid import uuid4

try:
    xrange
except NameError:
    xrange = range


def _identity(o):
    '''
    Return input unchanged (identity function).

    Args:
        o (object): anything

    Returns:
        `o`
    '''
    return o


def read_communication_from_redis_key(redis_db, key, add_references=True):
    '''
    Return a serialized communication from a string key.  If block
    is True, poll server until key appears at specified interval
    or until specified timeout (indefinitely if timeout is zero).
    Return None if block is False and key does not exist or if
    block is True and key does not exist after specified timeout.

    Args:
        redis_db (redis.Redis): Redis database connection object
        key (str): simple (string) key to read serialized
            communication from
        add_references (bool): If True, calls
           :func:`concrete.util.references.add_references_to_communication`
           on :class:`.Communication` read from file
    '''
    buf = redis_db.get(key)
    if buf is None:
        return None
    else:
        return read_communication_from_buffer(buf,
                                              add_references=add_references)


class RedisReader(object):
    '''
    Iterable class for reading one or more objects from redis.

    Supported input types are:

    - a set containing zero or more objects
    - a list containing zero or more objects
    - a hash containing zero or more key-object pairs

    For list and set types, the reader can optionally pop (consume) its
    input; for lists only, the reader can moreover block on the input.

    Note that iteration over a set or hash will create a temporary key
    in the redis database to maintain a set of elements scanned so far.

    If pop is False and the key (in the database) is modified during
    iteration, behavior is undefined.  If pop is True, modifications
    during iteration are encouraged.

    Example usage::

        from redis import Redis
        redis_db = Redis(port=12345)
        for obj in RedisReader(redis_db, 'my-set-key'):
            do_something(obj)
    '''

    def __init__(self, redis_db, key, key_type=None, pop=False, block=False,
                 right_to_left=True,
                 block_timeout=0, temp_key_ttl=3600, temp_key_leaf_len=32,
                 cycle_list=False, deserialize_func=None):
        '''
        Create reader for specified key in specified
        redis_db.

        Args:
            redis_db (redis.Redis): Redis database connection object
            key (str): name of redis key containing your object(s)
            key_type (str): 'set', 'list', 'hash', or None; if None,
                look up type in redis (only works if the key exists, so
                probably not suitable for block and/or pop modes)
            pop (bool): True to remove objects from redis
                as we iterate over them, and False to leave redis
                unaltered
            block (bool): True to block for data (i.e., wait for
                something to be added to the list if it is empty),
                False to end iteration when there is no more data
            right_to_left (bool): True to iterate over and index in
                lists from right to left, False to iterate/index
                from left to right
            deserialize_func (func): maps blobs from redis to some more
                friendly representation (e.g., if all your items
                are unicode strings, you might want to specify
                lambda s: s.decode('utf-8')); return blobs
                unchanged if deserialize_func is None
            block_timeout (int): number of seconds to block during
                operations if `block` is True; if 0, block forever
            temp_key_ttl (int): time-to-live (in seconds) of temporary
                keys created during scans (amount of time to process
                a batch of items returned by a scan should be much less
                than the time-to-live of the temporary key, or duplicate
                items will be returned)
            temp_key_leaf_len (int): length (in bytes) of random
                part of temporary key (longer is less likely to cause
                conflicts with other processes but slower)
            cycle_list (bool): iterate over list by popping items from
                the right end and pushing them onto the left end
                (atomically), note iteration thus modifies the list
                (although a full iteration ultimately leaves the list in
                the same state as it began)

        Raises:
            Exception: if `key_type` is None but the key does not exist
                in the database (so its type cannot be guessed)
            ValueError: if key type is not recognized or the options
                that were specified are not supported for a recognized
                key type
        '''
        self.redis_db = redis_db
        self.key = key

        if key_type is None:
            # try to guess type
            if redis_db.exists(key):
                key_type = redis_db.type(key).decode('utf-8')
            else:
                raise Exception('can only guess type of key that exists')

        if key_type not in ('set', 'hash', 'list'):
            raise ValueError('unrecognized key type %s' % key_type)

        self.key_type = key_type

        if pop and key_type not in ('set', 'list'):
            raise ValueError('can only pop on set or list')
        if block and key_type not in ('list',):
            raise ValueError('can only block-pop on list')
        if block and not pop:
            raise ValueError('can only block if popping too')
        if cycle_list and block:
            raise ValueError('can only cycle list when not blocking')
        if cycle_list and key_type not in ('list',):
            raise ValueError('can only cycle list on list type')
        if cycle_list and not right_to_left:
            raise ValueError('can only cycle list if processing right-to-left')

        self.pop = pop
        self.block = block
        self.block_timeout = block_timeout

        self.temp_key_ttl = temp_key_ttl
        self.temp_key_leaf_len = temp_key_leaf_len

        self.right_to_left = right_to_left

        self.cycle_list = cycle_list

        self.deserialize_func = (
            deserialize_func
            if (deserialize_func is not None)
            else _identity
        )

    def __iter__(self):
        '''
        Return iterator over respective redis data structure.

        Returns:
            iterator over data structure

        Raises:
            Exception: if key type is not recognized or unsupported
                options were used with a recognized key type
        '''
        if self.key_type in ('list', 'set') and self.pop:
            buf = self._pop_buf()
            while buf is not None:
                yield self.deserialize_func(buf)
                buf = self._pop_buf()

        elif self.key_type == 'list' and not self.pop:
            if self.cycle_list:
                buf = self.redis_db.rpoplpush(self.key, self.key)
                i = 0
                while buf is not None and i < self.redis_db.llen(self.key):
                    yield self.deserialize_func(buf)
                    buf = self.redis_db.rpoplpush(self.key, self.key)
                    i += 1
            else:
                for i in xrange(self.redis_db.llen(self.key)):
                    idx = -(i + 1) if self.right_to_left else i
                    buf = self.redis_db.lindex(self.key, idx)
                    yield self.deserialize_func(buf)

        elif self.key_type in ('set', 'hash') and not self.pop:
            if self.key_type == 'set':
                num_objs = self.redis_db.scard(self.key)
                scan = self.redis_db.sscan
                # batch is an iterable of buffers

                def _deserialize(k, batch):
                    return self.deserialize_func(k)
            else:
                num_objs = self.redis_db.hlen(self.key)
                scan = self.redis_db.hscan
                # batch is a dict of key-buffer pairs

                def _deserialize(k, batch):
                    return self.deserialize_func(batch[k])

            get_obj = _deserialize

            temp_key = self._make_temp_key()

            i = 0
            cursor = 0
            while i < num_objs:
                (cursor, batch) = scan(self.key, cursor)
                for k in batch:
                    if i == num_objs:
                        break
                    if self.redis_db.sadd(temp_key, k) > 0:
                        i += 1
                        yield get_obj(k, batch)
                    self.redis_db.expire(temp_key, self.temp_key_ttl)

        else:
            raise Exception('not implemented')

    def __len__(self):
        '''
        Return instantaneous length of dataset.

        Returns:
            instantaneous length of data set in redis (subject to change
            if database is modified)

        Raises:
            Exception: if called on redis key type that is not a
                list, hash, or set
        '''
        if self.key_type == 'list':
            return self.redis_db.llen(self.key)
        elif self.key_type == 'set':
            return self.redis_db.scard(self.key)
        elif self.key_type == 'hash':
            return self.redis_db.hlen(self.key)
        else:
            raise Exception('not implemented')

    def __getitem__(self, k):
        '''
        Return item at specified list index or hash key;
        never pop or block.

        Args:
            k: list index (int) or hash key (str)

        Raises:
            Exception: if called on redis key type that is not a
                list or hash
        '''
        if self.key_type in ('list', 'hash'):
            if self.key_type == 'list':
                idx = -(k + 1) if self.right_to_left else k
                buf = self.redis_db.lindex(self.key, idx)
            else:
                buf = self.redis_db.hget(self.key, k)
            return None if buf is None else self.deserialize_func(buf)

        else:
            raise Exception('not implemented')

    def batch(self, n):
        '''
        Return a batch of n objects.  May be faster than
        one-at-a-time iteration, but currently only supported for
        non-popping, non-blocking set configurations.  Support for
        popping, non-blocking sets is planned; see
        http://redis.io/commands/spop .

        Args:
            n (int): number of objects to return

        Raises:
            Exception: if key type is not a set, or if it is a set
                but popping or blocking operation is specified
        '''
        if self.key_type == 'set' and not self.pop and not self.block:
            return [
                self.deserialize_func(buf)
                for buf
                in self.redis_db.srandmember(self.key, n)
            ]

        else:
            raise Exception('not implemented')

    def _pop_buf(self):
        '''
        Pop and return a serialized object, or None if there is
        none (or we blocked and timed out).

        Raises:
            Exception: if called on redis key type that is not a
                list or set, or if the key type is a set and blocking
                operation is specified
        '''
        if self.key_type == 'list':
            if self.block:
                _pop = (
                    self.redis_db.brpop
                    if self.right_to_left
                    else self.redis_db.blpop
                )
                val = _pop(self.key, timeout=self.block_timeout)
                return None if val is None else val[1]
            else:
                _pop = (
                    self.redis_db.rpop
                    if self.right_to_left
                    else self.redis_db.lpop
                )
                return _pop(self.key)

        elif self.key_type == 'set' and not self.block:
            return self.redis_db.spop(self.key)

        else:
            raise Exception('not implemented')

    def _make_temp_key(self):
        '''
        Generate temporary (random, unused) key.  Do not set in redis
        (leave that to the caller).

        Returns:
            random unused redis key (note: could be no longer unused
            by the time the caller reads it, however unlikely)
        '''
        temp_key = None
        while temp_key is None or self.redis_db.exists(temp_key):
            # thanks: http://stackoverflow.com/a/2257449
            temp_key_leaf = str(uuid4())
            temp_key = ':'.join(('temp', self.key, temp_key_leaf))
        return temp_key

    def __str__(self):
        return '%s(%s, %s, %s)' % (type(self).__name__, self.redis_db,
                                   self.key, self.key_type)


class RedisCommunicationReader(RedisReader):
    '''
    Iterable class for reading one or more Communications from redis.
    See RedisReader for further description.

    Example usage::

        from redis import Redis
        redis_db = Redis(port=12345)
        for comm in RedisCommunicationReader(redis_db, 'my-set-key'):
            do_something(comm)
    '''

    def __init__(self, redis_db, key, add_references=True, **kwargs):
        '''
        Create communication reader for specified key in specified
        redis_db.

        Args:
            redis_db (redis.Redis): Redis database connection object
            key (str): name of redis key containing your
                communication(s)
            add_references (bool): True to fill in members in the
                communication according to UUID relationships (see
                concrete.util.add_references), False to return
                communication as-is (note: you may need this False
                if you are dealing with incomplete communications)

        All other keyword arguments are passed through to RedisReader;
        see :class:`.RedisReader` for a description of those
        arguments.

        Raises:
            Exception: if `deserialize_func` is specified (it is set
                to the appropriate concrete deserializer internally)
        '''

        if 'deserialize_func' in kwargs:
            raise Exception('RedisCommunicationReader does not allow custom '
                            'deserialize_func')
        self.add_references = add_references
        super(RedisCommunicationReader, self).__init__(
            redis_db, key, deserialize_func=self._load_from_buffer, **kwargs
        )

    def _load_from_buffer(self, buf):
        '''
        Deserialize communication from string buffer and return.

        Args:
            buf (str): buffer containing serialized communication

        Returns:
            Communication deserialized from `buf`
        '''
        return read_communication_from_buffer(
            buf, add_references=self.add_references)


def write_communication_to_redis_key(redis_db, key, comm):
    '''
    Serialize communication and store result in redis key.

    Args:
        redis_db (redis.Redis): Redis database connection object
        key (str): name of simple (string) redis key to write
            communication to
        comm (Communication): communication to serialize
    '''
    redis_db.set(key, write_communication_to_buffer(comm))


class RedisWriter(object):
    '''
    Class for writing one or more objects to redis.

    Supported input types are:

    - a set of objects
    - a list of objects
    - a hash of key-object pairs

    Example usage:

        from redis import Redis
        redis_db = Redis(port=12345)
        w = RedisWriter(redis_db, 'my-set-key')
        w.write(obj)
    '''

    def __init__(self, redis_db, key, key_type=None, right_to_left=True,
                 serialize_func=None, hash_key_func=None):
        '''
        Create object writer for specified key in specified
        redis_db.

        Args:
            redis_db (redis.Redis): Redis database connection object
            key (str): name of redis key containing your object(s)
            key_type (str): 'set', 'list', 'hash', or None; if None,
                look up type in redis (only works if the key exists)
            right_to_left (bool): True to write elements to the left
                end of lists, False to write to the right end
            serialize_func (func): maps objects to blobs before
                sending to Redis (e.g., if everything you write
                will be a unicode string, you might want to use
                lambda u: u.encode('utf-8')); pass objects to
                Redis unchanged if serialize_func is None
            hash_key_func (func): maps objects to keys when key_type
                is hash (None: use Python's hash function)
        '''
        self.redis_db = redis_db
        self.key = key

        if key_type is None:
            # try to guess type
            if redis_db.exists(key):
                key_type = redis_db.type(key).decode('utf-8')
            else:
                raise ValueError('can only guess type of key that exists')

        if key_type not in ('set', 'hash', 'list'):
            raise ValueError('unrecognized key type %s' % key_type)

        self.key_type = key_type
        self.right_to_left = right_to_left

        self.serialize_func = (
            serialize_func
            if (serialize_func is not None)
            else _identity
        )
        self.hash_key_func = (
            hash_key_func
            if (hash_key_func is not None)
            else hash
        )

    def clear(self):
        '''
        Remove all data from redis data structure.
        '''
        self.redis_db.delete(self.key)

    def write(self, obj):
        '''
        Write object obj to redis data structure.

        Args:
            obj (object): object to be serialized by
            `self.serialize_func` and written to database, according
            to key type

        Raises:
            Exception: if called on redis key type that is not a
                list, set, or hash
        '''
        buf = self.serialize_func(obj)

        if self.key_type == 'list':
            _push = (
                self.redis_db.lpush
                if self.right_to_left
                else self.redis_db.rpush
            )
            return _push(self.key, buf)

        elif self.key_type == 'set':
            return self.redis_db.sadd(self.key, buf)

        elif self.key_type == 'hash':
            return self.redis_db.hset(self.key, self.hash_key_func(obj), buf)

        else:
            raise Exception('not implemented')

    def __str__(self):
        return '%s(%s, %s, %s)' % (type(self).__name__, self.redis_db,
                                   self.key, self.key_type)


class RedisCommunicationWriter(RedisWriter):
    '''
    Class for writing one or more Communications to redis.
    See RedisWriter for further description.

    Example usage:

        from redis import Redis
        redis_db = Redis(port=12345)
        w = RedisCommunicationWriter(redis_db, 'my-set-key')
        w.write(comm)
    '''

    def __init__(self, redis_db, key, uuid_hash_key=False, **kwargs):
        '''
        Create communication writer for specified key in specified
        redis_db.

        Args:
            redis_db (redis.Redis): Redis database connection object
            key (str): name of redis key containing your
                communication(s)
            uuid_hash_key (bool): True to use the UUID as the hash key
                 for a communication, False to use the id

        All other keyword arguments are passed through to RedisWriter;
        see :class:`.RedisWriter` for a description of those
        arguments.

        Raises:
            Exception: if `serialize_func` is specified (it is set
                to the appropriate concrete serializer internally),
                or if `hash_key_func` is specified (it is set to an
                appropriate function internally)
        '''
        if 'serialize_func' in kwargs:
            raise Exception('RedisCommunicationWriter does not allow custom '
                            'serialize_func')
        if 'hash_key_func' in kwargs:
            raise Exception('RedisCommunicationWriter does not allow custom '
                            'hash_key_func')
        self.uuid_hash_key = uuid_hash_key
        super(RedisCommunicationWriter, self).__init__(
            redis_db, key, serialize_func=self._write_to_buffer,
            hash_key_func=self._to_key, **kwargs
        )

    def _write_to_buffer(self, comm):
        '''
        Serialize communication and return result as a string.

        Args:
            comm (Communication): communication to serialize

        Returns:
            string representing serialized `comm`
        '''
        return write_communication_to_buffer(comm)

    def _to_key(self, comm):
        '''
        Return hash key of given communication.

        Args:
            comm (Communication): communication to return hash key for

        Returns:
            Communication UUID string if `self.uuid_hash_key` is True,
            Communication id otherwise.
        '''
        return (comm.uuid.uuidString if self.uuid_hash_key else comm.id)

    def __str__(self):
        return '%s(%s, %s, %s)' % (type(self).__name__, self.redis_db,
                                   self.key, self.key_type)



from __future__ import unicode_literals
import json

from pytest import fixture, mark

from concrete.util import (
    json_tweet_object_to_Communication, twitter_lid_to_iso639_3)


TWEET_TXT = ("Barber tells me - his son is colorblind / my hair is auburn /"
             " and auburn is a shade of green")

TWEET_ID = 238426131689242624
TWEET_ID_STR = u'238426131689242624'

RT_TWEET_TXT = 'Barber tells me'

RT_TWEET_ID = 238426131689242623
RT_TWEET_ID_STR = u'238426131689242623'

REPLY_TWEET_ID = 238426131689242622
REPLY_TWEET_ID_STR = u'238426131689242622'


@fixture
def tweet():
    return {u'text': TWEET_TXT,
            u'id_str': TWEET_ID_STR,
            u'id': TWEET_ID,
            u'created_at': 'Wed Aug 27 13:08:45 +0000 2008',
            u'user': {u'screen_name': 'charman',
                      u'name': 'C Harman',
                      u'lang': 'ja',
                      u'verified': False,
                      u'id': 1234,
                      u'id_str': '1234',
                      u'geo_enabled': True,
                      u'created_at': 'Wed Aug 27 13:08:05 +0000 2008',
                      u'friends_count': 37,
                      u'statuses_count': 38,
                      u'listed_count': 39,
                      u'favourites_count': 41,
                      u'followers_count': 42,
                      u'location': 'San Francisco, CA',
                      u'time_zone': 'Pacific Time (US & Canada)',
                      u'description': 'The Real Twitter API.',
                      u'url': 'http://dev.twitter.com',
                      u'utc_offset': -18000},
            u'entities': {u'hashtags': [{u'indices': [32, 36],
                                         u'text': u'lol'}],
                          u'user_mentions': [{u'name': u'Twitter API',
                                              u'indices': [4, 15],
                                              u'screen_name': u'twitterapi',
                                              u'id': 6253282,
                                              u'id_str': u'6253282'}],
                          u'urls': [{u'indices': [32, 52],
                                     u'url': u'http://t.co/IOwBrTZR',
                                     u'display_url': u'youtube.com/watch?'
                                                     u'v=oHg5SJ\u2026',
                                     u'expanded_url': u'http://www.youtube.'
                                                      u'com/watch?'
                                                      u'v=oHg5SJYRHA0'}]},
            u'coordinates': {u'coordinates': [-75.5, 40.25],
                             u'type': u'Point'},
            u'place': {u'attributes': {u'street_address': u'795 Folsom St',
                                       u'region': u'Mid Atlantic',
                                       u'locality': u'Washington, DC'},
                       u'bounding_box': {u'coordinates': [[[-77.25, 38.5],
                                                           [-76.0, 38.5],
                                                           [-76.0, 38.125],
                                                           [-77.25, 38.125]]],
                                         u'type': u'Polygon'},
                       u'country': u'United States',
                       u'country_code': u'US',
                       u'full_name': u'Washington, DC',
                       u'id': u'01fbe706f872cb32',
                       u'name': u'Washington',
                       u'place_type': u'city',
                       u'url': u'http://api.twitter.com/1/geo/id/'
                               u'01fbe706f872cb32.json'},
            u'retweeted_status': {u'text': RT_TWEET_TXT,
                                  u'id_str': RT_TWEET_ID_STR,
                                  u'id': RT_TWEET_ID,
                                  u'created_at': 'Wed Aug 27 13:08:44 +0000 '
                                                 '2008',
                                  u'user': {u'screen_name': 'charman2',
                                            u'name': 'C Harman 2',
                                            u'lang': 'ja',
                                            u'verified': False,
                                            u'id': 1235,
                                            u'id_str': '1235'},
                                  u'entities': {u'hashtags': [],
                                                u'user_mentions': [],
                                                u'urls': []}},
            u'in_reply_to_status_id': REPLY_TWEET_ID,
            u'in_reply_to_status_id_str': REPLY_TWEET_ID_STR,
            u'in_reply_to_screen_name': 'charman3',
            u'in_reply_to_user_id': 1236,
            u'in_reply_to_user_id_str': u'1236',
            u'retweeted': False,
            u'retweet_count': 1585,
            u'truncated': True,
            u'source': u'<a href="http://itunes.apple.com/us/app/twitter/'
                       u'id409789998?mt=12" >Twitter for Mac</a>',
            u'lang': u'en'}


def test_twitter_lid_conversion():
    assert 'eng' == twitter_lid_to_iso639_3('en-gb')
    assert 'msa' == twitter_lid_to_iso639_3('msa')
    assert 'hun' == twitter_lid_to_iso639_3('hu')
    assert 'zho' == twitter_lid_to_iso639_3('zh-cn')
    assert 'zho' == twitter_lid_to_iso639_3('zh-tw')
    assert 'und' == twitter_lid_to_iso639_3('und')
    assert 'eng' == twitter_lid_to_iso639_3('eng')
    # Three-letter language codes are not validated as ISO-639-3 codes
    assert 'xxx' == twitter_lid_to_iso639_3('xxx')
    # Two-letter language codes are validated
    assert 'und' == twitter_lid_to_iso639_3('xx')
    # Special handling for Indonesian, due to Twitter using incorrect ISO-639-1 code
    assert 'ind' == twitter_lid_to_iso639_3('in')


@mark.parametrize('omitted_fields,omitted_assertions', [
    ((), ()),
    (('lang',), ('lid',)),
    (('coordinates',), ('coordinates',)),
    (('place',), ('place',)),
    (('retweeted_status',), ('retweet',)),
    (('in_reply_to_status_id', 'in_reply_to_status_id_str',
      'in_reply_to_user_id', 'in_reply_to_user_id_str',
      'in_reply_to_screen_name',), ('reply',)),
])
def test_json_tweet_object_to_Communication(tweet, omitted_fields,
                                            omitted_assertions):
    for field in omitted_fields:
        del tweet[field]

    comm = json_tweet_object_to_Communication(tweet)
    tweet_info = comm.communicationMetadata.tweetInfo

    omitted_assertions = set(omitted_assertions)

    assert TWEET_ID_STR == comm.id
    assert TWEET_TXT == comm.text
    assert tweet == json.loads(comm.originalText)
    assert 1219842525 == comm.startTime
    assert 1219842525 == comm.endTime

    assert TWEET_ID == tweet_info.id
    assert TWEET_TXT == tweet_info.text
    assert 'Wed Aug 27 13:08:45 +0000 2008' == tweet_info.createdAt
    assert tweet_info.retweeted is False
    assert 1585 == tweet_info.retweetCount
    assert tweet_info.truncated is True
    assert u'<a href="http://itunes.apple.com/us/app/twitter/' \
           u'id409789998?mt=12" >Twitter for Mac</a>' == tweet_info.source

    assert 'jpn' == tweet_info.user.lang
    assert 'charman' == tweet_info.user.screenName
    assert 'C Harman' == tweet_info.user.name
    assert 1234 == tweet_info.user.id
    assert tweet_info.user.geoEnabled is True
    assert 'Wed Aug 27 13:08:05 +0000 2008' == tweet_info.user.createdAt
    assert 37 == tweet_info.user.friendsCount
    assert 38 == tweet_info.user.statusesCount
    assert 39 == tweet_info.user.listedCount
    assert 41 == tweet_info.user.favouritesCount
    assert 42 == tweet_info.user.followersCount
    assert 'San Francisco, CA' == tweet_info.user.location
    assert 'Pacific Time (US & Canada)' == tweet_info.user.timeZone
    assert 'The Real Twitter API.' == tweet_info.user.description
    assert 'http://dev.twitter.com' == tweet_info.user.url
    assert -18000 == tweet_info.user.utcOffset

    if 'retweet' in omitted_assertions:
        omitted_assertions.remove('retweet')
        assert tweet_info.retweetedStatusId is None
        assert tweet_info.retweetedUserId is None
        assert tweet_info.retweetedScreenName is None
    else:
        assert RT_TWEET_ID == tweet_info.retweetedStatusId
        assert 1235 == tweet_info.retweetedUserId
        assert 'charman2' == tweet_info.retweetedScreenName

    if 'reply' in omitted_assertions:
        omitted_assertions.remove('reply')
        assert tweet_info.inReplyToStatusId is None
        assert tweet_info.inReplyToUserId is None
        assert tweet_info.inReplyToScreenName is None
    else:
        assert REPLY_TWEET_ID == tweet_info.inReplyToStatusId
        assert 1236 == tweet_info.inReplyToUserId
        assert 'charman3' == tweet_info.inReplyToScreenName

    if 'entities' in omitted_assertions:
        omitted_assertions.remove('entities')
        assert tweet_info.entities is None
    else:
        assert 1 == len(tweet_info.entities.hashtagList)
        assert u'lol' == tweet_info.entities.hashtagList[0].text
        assert 32 == tweet_info.entities.hashtagList[0].startOffset
        assert 36 == tweet_info.entities.hashtagList[0].endOffset
        assert 1 == len(tweet_info.entities.userMentionList)
        assert 6253282 == tweet_info.entities.userMentionList[0].id
        assert u'Twitter API' == tweet_info.entities.userMentionList[0].name
        assert u'twitterapi' == \
            tweet_info.entities.userMentionList[0].screenName
        assert 4 == tweet_info.entities.userMentionList[0].startOffset
        assert 15 == tweet_info.entities.userMentionList[0].endOffset
        assert 1 == len(tweet_info.entities.urlList)
        assert u'http://t.co/IOwBrTZR' == \
            tweet_info.entities.urlList[0].url
        assert u'youtube.com/watch?v=oHg5SJ\u2026' == \
            tweet_info.entities.urlList[0].displayUrl
        assert u'http://www.youtube.com/watch?v=oHg5SJYRHA0' == \
            tweet_info.entities.urlList[0].expandedUrl
        assert 32 == tweet_info.entities.urlList[0].startOffset
        assert 52 == tweet_info.entities.urlList[0].endOffset

    if 'coordinates' in omitted_assertions:
        omitted_assertions.remove('coordinates')
        assert tweet_info.coordinates is None
    else:
        assert u'Point' == tweet_info.coordinates.type
        assert -75.5 == tweet_info.coordinates.coordinates.longitude
        assert 40.25 == tweet_info.coordinates.coordinates.latitude

    if 'place' in omitted_assertions:
        omitted_assertions.remove('place')
        assert tweet_info.place is None
    else:
        assert u'Mid Atlantic' == tweet_info.place.attributes.region
        assert u'Washington, DC' == tweet_info.place.attributes.locality
        assert u'795 Folsom St' == tweet_info.place.attributes.streetAddress
        assert u'Polygon' == tweet_info.place.boundingBox.type
        assert -77.25 == \
            tweet_info.place.boundingBox.coordinateList[0].longitude
        assert 38.5 == \
            tweet_info.place.boundingBox.coordinateList[0].latitude
        assert -76.0 == \
            tweet_info.place.boundingBox.coordinateList[1].longitude
        assert 38.5 == \
            tweet_info.place.boundingBox.coordinateList[1].latitude
        assert -76.0 == \
            tweet_info.place.boundingBox.coordinateList[2].longitude
        assert 38.125 == \
            tweet_info.place.boundingBox.coordinateList[2].latitude
        assert -77.25 == \
            tweet_info.place.boundingBox.coordinateList[3].longitude
        assert 38.125 == \
            tweet_info.place.boundingBox.coordinateList[3].latitude
        assert u'United States' == tweet_info.place.country
        assert u'US' == tweet_info.place.countryCode
        assert u'Washington, DC' == tweet_info.place.fullName
        assert u'01fbe706f872cb32' == tweet_info.place.id
        assert u'Washington' == tweet_info.place.name
        assert u'city' == tweet_info.place.placeType
        assert u'http://api.twitter.com/1/geo/id/01fbe706f872cb32.json' == \
            tweet_info.place.url

    if 'lid' in omitted_assertions:
        omitted_assertions.remove('lid')
        assert comm.lidList is None
    else:
        assert 1 == len(comm.lidList)
        kvm = comm.lidList[0].languageToProbabilityMap
        assert set(['eng']) == set(kvm.keys())
        assert 1.0 == kvm['eng']

    assert not omitted_assertions



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.uuid.ttypes
import concrete.twitter.ttypes
import concrete.email.ttypes
import concrete.nitf.ttypes

from thrift.transport import TTransport
all_structs = []


class TheoryDependencies(object):
    """
    A struct that holds UUIDs for all theories that a particular
    annotation was based upon (and presumably requires).

    Producers of TheoryDependencies should list all stages that they
    used in constructing their particular annotation. They do not,
    however, need to explicitly label *each* stage; they can label
    only the immediate stage before them.

    Examples:

    If you are producing a Tokenization, and only used the
    SentenceSegmentation in order to produce that Tokenization, list
    only the single SentenceSegmentation UUID in sentenceTheoryList.

    In this example, even though the SentenceSegmentation will have
    a dependency on some SectionSegmentation, it is not necessary
    for the Tokenization to list the SectionSegmentation UUID as a
    dependency.

    If you are a producer of EntityMentions, and you use two
    POSTokenTagging and one NERTokenTagging objects, add the UUIDs for
    the POSTokenTagging objects to posTagTheoryList, and the UUID of
    the NER TokenTagging to the nerTagTheoryList.

    In this example, because multiple annotations influenced the
    new annotation, they should all be listed as dependencies.

    Attributes:
     - sectionTheoryList
     - sentenceTheoryList
     - tokenizationTheoryList
     - posTagTheoryList
     - nerTagTheoryList
     - lemmaTheoryList
     - langIdTheoryList
     - parseTheoryList
     - dependencyParseTheoryList
     - tokenAnnotationTheoryList
     - entityMentionSetTheoryList
     - entitySetTheoryList
     - situationMentionSetTheoryList
     - situationSetTheoryList
     - communicationsList

    """


    def __init__(self, sectionTheoryList=None, sentenceTheoryList=None, tokenizationTheoryList=None, posTagTheoryList=None, nerTagTheoryList=None, lemmaTheoryList=None, langIdTheoryList=None, parseTheoryList=None, dependencyParseTheoryList=None, tokenAnnotationTheoryList=None, entityMentionSetTheoryList=None, entitySetTheoryList=None, situationMentionSetTheoryList=None, situationSetTheoryList=None, communicationsList=None,):
        self.sectionTheoryList = sectionTheoryList
        self.sentenceTheoryList = sentenceTheoryList
        self.tokenizationTheoryList = tokenizationTheoryList
        self.posTagTheoryList = posTagTheoryList
        self.nerTagTheoryList = nerTagTheoryList
        self.lemmaTheoryList = lemmaTheoryList
        self.langIdTheoryList = langIdTheoryList
        self.parseTheoryList = parseTheoryList
        self.dependencyParseTheoryList = dependencyParseTheoryList
        self.tokenAnnotationTheoryList = tokenAnnotationTheoryList
        self.entityMentionSetTheoryList = entityMentionSetTheoryList
        self.entitySetTheoryList = entitySetTheoryList
        self.situationMentionSetTheoryList = situationMentionSetTheoryList
        self.situationSetTheoryList = situationSetTheoryList
        self.communicationsList = communicationsList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.sectionTheoryList = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = concrete.uuid.ttypes.UUID()
                        _elem5.read(iprot)
                        self.sectionTheoryList.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.sentenceTheoryList = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = concrete.uuid.ttypes.UUID()
                        _elem11.read(iprot)
                        self.sentenceTheoryList.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.tokenizationTheoryList = []
                    (_etype15, _size12) = iprot.readListBegin()
                    for _i16 in range(_size12):
                        _elem17 = concrete.uuid.ttypes.UUID()
                        _elem17.read(iprot)
                        self.tokenizationTheoryList.append(_elem17)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.posTagTheoryList = []
                    (_etype21, _size18) = iprot.readListBegin()
                    for _i22 in range(_size18):
                        _elem23 = concrete.uuid.ttypes.UUID()
                        _elem23.read(iprot)
                        self.posTagTheoryList.append(_elem23)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.nerTagTheoryList = []
                    (_etype27, _size24) = iprot.readListBegin()
                    for _i28 in range(_size24):
                        _elem29 = concrete.uuid.ttypes.UUID()
                        _elem29.read(iprot)
                        self.nerTagTheoryList.append(_elem29)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.LIST:
                    self.lemmaTheoryList = []
                    (_etype33, _size30) = iprot.readListBegin()
                    for _i34 in range(_size30):
                        _elem35 = concrete.uuid.ttypes.UUID()
                        _elem35.read(iprot)
                        self.lemmaTheoryList.append(_elem35)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.LIST:
                    self.langIdTheoryList = []
                    (_etype39, _size36) = iprot.readListBegin()
                    for _i40 in range(_size36):
                        _elem41 = concrete.uuid.ttypes.UUID()
                        _elem41.read(iprot)
                        self.langIdTheoryList.append(_elem41)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.LIST:
                    self.parseTheoryList = []
                    (_etype45, _size42) = iprot.readListBegin()
                    for _i46 in range(_size42):
                        _elem47 = concrete.uuid.ttypes.UUID()
                        _elem47.read(iprot)
                        self.parseTheoryList.append(_elem47)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.LIST:
                    self.dependencyParseTheoryList = []
                    (_etype51, _size48) = iprot.readListBegin()
                    for _i52 in range(_size48):
                        _elem53 = concrete.uuid.ttypes.UUID()
                        _elem53.read(iprot)
                        self.dependencyParseTheoryList.append(_elem53)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 10:
                if ftype == TType.LIST:
                    self.tokenAnnotationTheoryList = []
                    (_etype57, _size54) = iprot.readListBegin()
                    for _i58 in range(_size54):
                        _elem59 = concrete.uuid.ttypes.UUID()
                        _elem59.read(iprot)
                        self.tokenAnnotationTheoryList.append(_elem59)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 11:
                if ftype == TType.LIST:
                    self.entityMentionSetTheoryList = []
                    (_etype63, _size60) = iprot.readListBegin()
                    for _i64 in range(_size60):
                        _elem65 = concrete.uuid.ttypes.UUID()
                        _elem65.read(iprot)
                        self.entityMentionSetTheoryList.append(_elem65)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 12:
                if ftype == TType.LIST:
                    self.entitySetTheoryList = []
                    (_etype69, _size66) = iprot.readListBegin()
                    for _i70 in range(_size66):
                        _elem71 = concrete.uuid.ttypes.UUID()
                        _elem71.read(iprot)
                        self.entitySetTheoryList.append(_elem71)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 13:
                if ftype == TType.LIST:
                    self.situationMentionSetTheoryList = []
                    (_etype75, _size72) = iprot.readListBegin()
                    for _i76 in range(_size72):
                        _elem77 = concrete.uuid.ttypes.UUID()
                        _elem77.read(iprot)
                        self.situationMentionSetTheoryList.append(_elem77)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 14:
                if ftype == TType.LIST:
                    self.situationSetTheoryList = []
                    (_etype81, _size78) = iprot.readListBegin()
                    for _i82 in range(_size78):
                        _elem83 = concrete.uuid.ttypes.UUID()
                        _elem83.read(iprot)
                        self.situationSetTheoryList.append(_elem83)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 15:
                if ftype == TType.LIST:
                    self.communicationsList = []
                    (_etype87, _size84) = iprot.readListBegin()
                    for _i88 in range(_size84):
                        _elem89 = concrete.uuid.ttypes.UUID()
                        _elem89.read(iprot)
                        self.communicationsList.append(_elem89)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TheoryDependencies')
        if self.sectionTheoryList is not None:
            oprot.writeFieldBegin('sectionTheoryList', TType.LIST, 1)
            oprot.writeListBegin(TType.STRUCT, len(self.sectionTheoryList))
            for iter90 in self.sectionTheoryList:
                iter90.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.sentenceTheoryList is not None:
            oprot.writeFieldBegin('sentenceTheoryList', TType.LIST, 2)
            oprot.writeListBegin(TType.STRUCT, len(self.sentenceTheoryList))
            for iter91 in self.sentenceTheoryList:
                iter91.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.tokenizationTheoryList is not None:
            oprot.writeFieldBegin('tokenizationTheoryList', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.tokenizationTheoryList))
            for iter92 in self.tokenizationTheoryList:
                iter92.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.posTagTheoryList is not None:
            oprot.writeFieldBegin('posTagTheoryList', TType.LIST, 4)
            oprot.writeListBegin(TType.STRUCT, len(self.posTagTheoryList))
            for iter93 in self.posTagTheoryList:
                iter93.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.nerTagTheoryList is not None:
            oprot.writeFieldBegin('nerTagTheoryList', TType.LIST, 5)
            oprot.writeListBegin(TType.STRUCT, len(self.nerTagTheoryList))
            for iter94 in self.nerTagTheoryList:
                iter94.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.lemmaTheoryList is not None:
            oprot.writeFieldBegin('lemmaTheoryList', TType.LIST, 6)
            oprot.writeListBegin(TType.STRUCT, len(self.lemmaTheoryList))
            for iter95 in self.lemmaTheoryList:
                iter95.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.langIdTheoryList is not None:
            oprot.writeFieldBegin('langIdTheoryList', TType.LIST, 7)
            oprot.writeListBegin(TType.STRUCT, len(self.langIdTheoryList))
            for iter96 in self.langIdTheoryList:
                iter96.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.parseTheoryList is not None:
            oprot.writeFieldBegin('parseTheoryList', TType.LIST, 8)
            oprot.writeListBegin(TType.STRUCT, len(self.parseTheoryList))
            for iter97 in self.parseTheoryList:
                iter97.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.dependencyParseTheoryList is not None:
            oprot.writeFieldBegin('dependencyParseTheoryList', TType.LIST, 9)
            oprot.writeListBegin(TType.STRUCT, len(self.dependencyParseTheoryList))
            for iter98 in self.dependencyParseTheoryList:
                iter98.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.tokenAnnotationTheoryList is not None:
            oprot.writeFieldBegin('tokenAnnotationTheoryList', TType.LIST, 10)
            oprot.writeListBegin(TType.STRUCT, len(self.tokenAnnotationTheoryList))
            for iter99 in self.tokenAnnotationTheoryList:
                iter99.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.entityMentionSetTheoryList is not None:
            oprot.writeFieldBegin('entityMentionSetTheoryList', TType.LIST, 11)
            oprot.writeListBegin(TType.STRUCT, len(self.entityMentionSetTheoryList))
            for iter100 in self.entityMentionSetTheoryList:
                iter100.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.entitySetTheoryList is not None:
            oprot.writeFieldBegin('entitySetTheoryList', TType.LIST, 12)
            oprot.writeListBegin(TType.STRUCT, len(self.entitySetTheoryList))
            for iter101 in self.entitySetTheoryList:
                iter101.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.situationMentionSetTheoryList is not None:
            oprot.writeFieldBegin('situationMentionSetTheoryList', TType.LIST, 13)
            oprot.writeListBegin(TType.STRUCT, len(self.situationMentionSetTheoryList))
            for iter102 in self.situationMentionSetTheoryList:
                iter102.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.situationSetTheoryList is not None:
            oprot.writeFieldBegin('situationSetTheoryList', TType.LIST, 14)
            oprot.writeListBegin(TType.STRUCT, len(self.situationSetTheoryList))
            for iter103 in self.situationSetTheoryList:
                iter103.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.communicationsList is not None:
            oprot.writeFieldBegin('communicationsList', TType.LIST, 15)
            oprot.writeListBegin(TType.STRUCT, len(self.communicationsList))
            for iter104 in self.communicationsList:
                iter104.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Digest(object):
    """
    Analytic-specific information about an attribute or edge. Digests
    are used to combine information from multiple sources to generate a
    unified value. The digests generated by an analytic will only ever
    be used by that same analytic, so analytics can feel free to encode
    information in whatever way is convenient.

    Attributes:
     - bytesValue: The following fields define various ways you can store the
    digest data (for convenience). If none of these meets your
    needs, then serialize the digest to a byte sequence and store it
    in bytesValue.
     - int64Value
     - doubleValue
     - stringValue
     - int64List
     - doubleList
     - stringList

    """


    def __init__(self, bytesValue=None, int64Value=None, doubleValue=None, stringValue=None, int64List=None, doubleList=None, stringList=None,):
        self.bytesValue = bytesValue
        self.int64Value = int64Value
        self.doubleValue = doubleValue
        self.stringValue = stringValue
        self.int64List = int64List
        self.doubleList = doubleList
        self.stringList = stringList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.bytesValue = iprot.readBinary()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I64:
                    self.int64Value = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.DOUBLE:
                    self.doubleValue = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.stringValue = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.int64List = []
                    (_etype108, _size105) = iprot.readListBegin()
                    for _i109 in range(_size105):
                        _elem110 = iprot.readI64()
                        self.int64List.append(_elem110)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.LIST:
                    self.doubleList = []
                    (_etype114, _size111) = iprot.readListBegin()
                    for _i115 in range(_size111):
                        _elem116 = iprot.readDouble()
                        self.doubleList.append(_elem116)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.LIST:
                    self.stringList = []
                    (_etype120, _size117) = iprot.readListBegin()
                    for _i121 in range(_size117):
                        _elem122 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        self.stringList.append(_elem122)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Digest')
        if self.bytesValue is not None:
            oprot.writeFieldBegin('bytesValue', TType.STRING, 1)
            oprot.writeBinary(self.bytesValue)
            oprot.writeFieldEnd()
        if self.int64Value is not None:
            oprot.writeFieldBegin('int64Value', TType.I64, 2)
            oprot.writeI64(self.int64Value)
            oprot.writeFieldEnd()
        if self.doubleValue is not None:
            oprot.writeFieldBegin('doubleValue', TType.DOUBLE, 3)
            oprot.writeDouble(self.doubleValue)
            oprot.writeFieldEnd()
        if self.stringValue is not None:
            oprot.writeFieldBegin('stringValue', TType.STRING, 4)
            oprot.writeString(self.stringValue.encode('utf-8') if sys.version_info[0] == 2 else self.stringValue)
            oprot.writeFieldEnd()
        if self.int64List is not None:
            oprot.writeFieldBegin('int64List', TType.LIST, 5)
            oprot.writeListBegin(TType.I64, len(self.int64List))
            for iter123 in self.int64List:
                oprot.writeI64(iter123)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.doubleList is not None:
            oprot.writeFieldBegin('doubleList', TType.LIST, 6)
            oprot.writeListBegin(TType.DOUBLE, len(self.doubleList))
            for iter124 in self.doubleList:
                oprot.writeDouble(iter124)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.stringList is not None:
            oprot.writeFieldBegin('stringList', TType.LIST, 7)
            oprot.writeListBegin(TType.STRING, len(self.stringList))
            for iter125 in self.stringList:
                oprot.writeString(iter125.encode('utf-8') if sys.version_info[0] == 2 else iter125)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class AnnotationMetadata(object):
    """
    Metadata associated with an annotation or a set of annotations,
    that identifies where those annotations came from.

    Attributes:
     - tool: The name of the tool that generated this annotation.
     - timestamp: The time at which this annotation was generated (in unix time
    UTC -- i.e., seconds since January 1, 1970).
     - digest: A Digest, carrying over any information the annotation metadata
    wishes to carry over.
     - dependencies: The theories that supported this annotation.

    An empty field indicates that the theory has no
    dependencies (e.g., an ingester).
     - kBest: An integer that represents a ranking for systems
    that output k-best lists.

    For systems that do not output k-best lists,
    the default value (1) should suffice.

    """


    def __init__(self, tool=None, timestamp=None, digest=None, dependencies=None, kBest=1,):
        self.tool = tool
        self.timestamp = timestamp
        self.digest = digest
        self.dependencies = dependencies
        self.kBest = kBest

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.tool = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I64:
                    self.timestamp = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRUCT:
                    self.digest = Digest()
                    self.digest.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRUCT:
                    self.dependencies = TheoryDependencies()
                    self.dependencies.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.I32:
                    self.kBest = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('AnnotationMetadata')
        if self.tool is not None:
            oprot.writeFieldBegin('tool', TType.STRING, 1)
            oprot.writeString(self.tool.encode('utf-8') if sys.version_info[0] == 2 else self.tool)
            oprot.writeFieldEnd()
        if self.timestamp is not None:
            oprot.writeFieldBegin('timestamp', TType.I64, 2)
            oprot.writeI64(self.timestamp)
            oprot.writeFieldEnd()
        if self.digest is not None:
            oprot.writeFieldBegin('digest', TType.STRUCT, 4)
            self.digest.write(oprot)
            oprot.writeFieldEnd()
        if self.dependencies is not None:
            oprot.writeFieldBegin('dependencies', TType.STRUCT, 5)
            self.dependencies.write(oprot)
            oprot.writeFieldEnd()
        if self.kBest is not None:
            oprot.writeFieldBegin('kBest', TType.I32, 6)
            oprot.writeI32(self.kBest)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.tool is None:
            raise TProtocolException(message='Required field tool is unset!')
        if self.timestamp is None:
            raise TProtocolException(message='Required field timestamp is unset!')
        if self.kBest is None:
            raise TProtocolException(message='Required field kBest is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class CommunicationMetadata(object):
    """
    Metadata specific to a particular Communication object.
    This might include corpus-specific metadata (from the Twitter API),
    attributes associated with the Communication (the author),
    or other information about the Communication.

    Attributes:
     - tweetInfo: Extra information for communications where kind==TWEET:
    Information about this tweet that is provided by the Twitter
    API.  For information about the Twitter API, see:
    https://dev.twitter.com/docs/platform-objects
     - emailInfo: Extra information for communications where kind==EMAIL
     - nitfInfo: Extra information that may come from the NITF
    (News Industry Text Format) schema. See 'nitf.thrift'.

    """


    def __init__(self, tweetInfo=None, emailInfo=None, nitfInfo=None,):
        self.tweetInfo = tweetInfo
        self.emailInfo = emailInfo
        self.nitfInfo = nitfInfo

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.tweetInfo = concrete.twitter.ttypes.TweetInfo()
                    self.tweetInfo.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.emailInfo = concrete.email.ttypes.EmailCommunicationInfo()
                    self.emailInfo.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRUCT:
                    self.nitfInfo = concrete.nitf.ttypes.NITFInfo()
                    self.nitfInfo.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('CommunicationMetadata')
        if self.tweetInfo is not None:
            oprot.writeFieldBegin('tweetInfo', TType.STRUCT, 1)
            self.tweetInfo.write(oprot)
            oprot.writeFieldEnd()
        if self.emailInfo is not None:
            oprot.writeFieldBegin('emailInfo', TType.STRUCT, 2)
            self.emailInfo.write(oprot)
            oprot.writeFieldEnd()
        if self.nitfInfo is not None:
            oprot.writeFieldBegin('nitfInfo', TType.STRUCT, 3)
            self.nitfInfo.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(TheoryDependencies)
TheoryDependencies.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'sectionTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 1
    (2, TType.LIST, 'sentenceTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 2
    (3, TType.LIST, 'tokenizationTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 3
    (4, TType.LIST, 'posTagTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 4
    (5, TType.LIST, 'nerTagTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 5
    (6, TType.LIST, 'lemmaTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 6
    (7, TType.LIST, 'langIdTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 7
    (8, TType.LIST, 'parseTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 8
    (9, TType.LIST, 'dependencyParseTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 9
    (10, TType.LIST, 'tokenAnnotationTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 10
    (11, TType.LIST, 'entityMentionSetTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 11
    (12, TType.LIST, 'entitySetTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 12
    (13, TType.LIST, 'situationMentionSetTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 13
    (14, TType.LIST, 'situationSetTheoryList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 14
    (15, TType.LIST, 'communicationsList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 15
)
all_structs.append(Digest)
Digest.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'bytesValue', 'BINARY', None, ),  # 1
    (2, TType.I64, 'int64Value', None, None, ),  # 2
    (3, TType.DOUBLE, 'doubleValue', None, None, ),  # 3
    (4, TType.STRING, 'stringValue', 'UTF8', None, ),  # 4
    (5, TType.LIST, 'int64List', (TType.I64, None, False), None, ),  # 5
    (6, TType.LIST, 'doubleList', (TType.DOUBLE, None, False), None, ),  # 6
    (7, TType.LIST, 'stringList', (TType.STRING, 'UTF8', False), None, ),  # 7
)
all_structs.append(AnnotationMetadata)
AnnotationMetadata.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'tool', 'UTF8', None, ),  # 1
    (2, TType.I64, 'timestamp', None, None, ),  # 2
    None,  # 3
    (4, TType.STRUCT, 'digest', [Digest, None], None, ),  # 4
    (5, TType.STRUCT, 'dependencies', [TheoryDependencies, None], None, ),  # 5
    (6, TType.I32, 'kBest', None, 1, ),  # 6
)
all_structs.append(CommunicationMetadata)
CommunicationMetadata.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'tweetInfo', [concrete.twitter.ttypes.TweetInfo, None], None, ),  # 1
    (2, TType.STRUCT, 'emailInfo', [concrete.email.ttypes.EmailCommunicationInfo, None], None, ),  # 2
    (3, TType.STRUCT, 'nitfInfo', [concrete.nitf.ttypes.NITFInfo, None], None, ),  # 3
)
fix_spec(all_structs)
del all_structs



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.uuid.ttypes
import concrete.metadata.ttypes

from thrift.transport import TTransport
all_structs = []


class ClusterMember(object):
    """
    An item being clustered. Does not designate cluster _membership_, as in
    "item x belongs to cluster C", but rather just the item ("x" in this
    example). Membership is indicated through Cluster objects.  An item may be a
    Entity, EntityMention, Situation, SituationMention, or technically anything
    with a UUID.

    Attributes:
     - communicationId: UUID of the Communication which contains the item specified by 'elementId'.
    This is ancillary info assuming UUIDs are indeed universally unique.
     - setId: UUID of the Entity|Situation(Mention)Set which contains the item specified by 'elementId'.
    This is ancillary info assuming UUIDs are indeed universally unique.
     - elementId: UUID of the EntityMention, Entity, SituationMention, or Situation that
    this item represents. This is the characteristic field.

    """


    def __init__(self, communicationId=None, setId=None, elementId=None,):
        self.communicationId = communicationId
        self.setId = setId
        self.elementId = elementId

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.communicationId = concrete.uuid.ttypes.UUID()
                    self.communicationId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.setId = concrete.uuid.ttypes.UUID()
                    self.setId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRUCT:
                    self.elementId = concrete.uuid.ttypes.UUID()
                    self.elementId.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('ClusterMember')
        if self.communicationId is not None:
            oprot.writeFieldBegin('communicationId', TType.STRUCT, 1)
            self.communicationId.write(oprot)
            oprot.writeFieldEnd()
        if self.setId is not None:
            oprot.writeFieldBegin('setId', TType.STRUCT, 2)
            self.setId.write(oprot)
            oprot.writeFieldEnd()
        if self.elementId is not None:
            oprot.writeFieldBegin('elementId', TType.STRUCT, 3)
            self.elementId.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.communicationId is None:
            raise TProtocolException(message='Required field communicationId is unset!')
        if self.setId is None:
            raise TProtocolException(message='Required field setId is unset!')
        if self.elementId is None:
            raise TProtocolException(message='Required field elementId is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Cluster(object):
    """
    A set of items which are alike in some way.  Has an implicit id which is the
    index of this Cluster in its parent Clustering's 'clusterList'.

    Attributes:
     - clusterMemberIndexList: The items in this cluster.  Values are indices into the
    'clusterMemberList' of the Clustering which contains this Cluster.
     - confidenceList: Co-indexed with 'clusterMemberIndexList'. The i^{th} value represents the
    confidence that mention clusterMemberIndexList[i] belongs to this cluster.
     - childIndexList: A set of clusters (implicit ids/indices) from which this cluster was
    created. This cluster should represent the union of all the items in all
    of the child clusters.  (For hierarchical clustering only).

    """


    def __init__(self, clusterMemberIndexList=None, confidenceList=None, childIndexList=None,):
        self.clusterMemberIndexList = clusterMemberIndexList
        self.confidenceList = confidenceList
        self.childIndexList = childIndexList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.clusterMemberIndexList = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = iprot.readI32()
                        self.clusterMemberIndexList.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.confidenceList = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = iprot.readDouble()
                        self.confidenceList.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.childIndexList = []
                    (_etype15, _size12) = iprot.readListBegin()
                    for _i16 in range(_size12):
                        _elem17 = iprot.readI32()
                        self.childIndexList.append(_elem17)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Cluster')
        if self.clusterMemberIndexList is not None:
            oprot.writeFieldBegin('clusterMemberIndexList', TType.LIST, 1)
            oprot.writeListBegin(TType.I32, len(self.clusterMemberIndexList))
            for iter18 in self.clusterMemberIndexList:
                oprot.writeI32(iter18)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.confidenceList is not None:
            oprot.writeFieldBegin('confidenceList', TType.LIST, 2)
            oprot.writeListBegin(TType.DOUBLE, len(self.confidenceList))
            for iter19 in self.confidenceList:
                oprot.writeDouble(iter19)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.childIndexList is not None:
            oprot.writeFieldBegin('childIndexList', TType.LIST, 3)
            oprot.writeListBegin(TType.I32, len(self.childIndexList))
            for iter20 in self.childIndexList:
                oprot.writeI32(iter20)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Clustering(object):
    """
    An (optionally) hierarchical clustering of items appearing across a set of
    Communications (intra-Communication clusterings are encoded by Entities and
    Situations).  An item may be a Entity, EntityMention, Situation,
    SituationMention, or technically anything with a UUID.

    Attributes:
     - uuid: UUID for this Clustering object.
     - metadata: Metadata for this Clustering object.
     - clusterMemberList: The set of items being clustered.
     - clusterList: Clusters of items. If this is a hierarchical clustering, this may contain
    clusters which are the set of smaller clusters.
    Clusters may not "overlap", meaning (for all clusters X,Y):
      X \cap Y \neq \emptyset \implies X \subset Y \vee Y \subset X
     - rootClusterIndexList: A set of disjoint clusters (indices in 'clusterList') which cover all
    items in 'clusterMemberList'. This list must be specified for hierarchical
    clusterings and should not be specified for flat clusterings.

    """


    def __init__(self, uuid=None, metadata=None, clusterMemberList=None, clusterList=None, rootClusterIndexList=None,):
        self.uuid = uuid
        self.metadata = metadata
        self.clusterMemberList = clusterMemberList
        self.clusterList = clusterList
        self.rootClusterIndexList = rootClusterIndexList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.clusterMemberList = []
                    (_etype24, _size21) = iprot.readListBegin()
                    for _i25 in range(_size21):
                        _elem26 = ClusterMember()
                        _elem26.read(iprot)
                        self.clusterMemberList.append(_elem26)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.clusterList = []
                    (_etype30, _size27) = iprot.readListBegin()
                    for _i31 in range(_size27):
                        _elem32 = Cluster()
                        _elem32.read(iprot)
                        self.clusterList.append(_elem32)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.rootClusterIndexList = []
                    (_etype36, _size33) = iprot.readListBegin()
                    for _i37 in range(_size33):
                        _elem38 = iprot.readI32()
                        self.rootClusterIndexList.append(_elem38)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Clustering')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 2)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.clusterMemberList is not None:
            oprot.writeFieldBegin('clusterMemberList', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.clusterMemberList))
            for iter39 in self.clusterMemberList:
                iter39.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.clusterList is not None:
            oprot.writeFieldBegin('clusterList', TType.LIST, 4)
            oprot.writeListBegin(TType.STRUCT, len(self.clusterList))
            for iter40 in self.clusterList:
                iter40.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.rootClusterIndexList is not None:
            oprot.writeFieldBegin('rootClusterIndexList', TType.LIST, 5)
            oprot.writeListBegin(TType.I32, len(self.rootClusterIndexList))
            for iter41 in self.rootClusterIndexList:
                oprot.writeI32(iter41)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.metadata is None:
            raise TProtocolException(message='Required field metadata is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(ClusterMember)
ClusterMember.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'communicationId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'setId', [concrete.uuid.ttypes.UUID, None], None, ),  # 2
    (3, TType.STRUCT, 'elementId', [concrete.uuid.ttypes.UUID, None], None, ),  # 3
)
all_structs.append(Cluster)
Cluster.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'clusterMemberIndexList', (TType.I32, None, False), None, ),  # 1
    (2, TType.LIST, 'confidenceList', (TType.DOUBLE, None, False), None, ),  # 2
    (3, TType.LIST, 'childIndexList', (TType.I32, None, False), None, ),  # 3
)
all_structs.append(Clustering)
Clustering.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 2
    (3, TType.LIST, 'clusterMemberList', (TType.STRUCT, [ClusterMember, None], False), None, ),  # 3
    (4, TType.LIST, 'clusterList', (TType.STRUCT, [Cluster, None], False), None, ),  # 4
    (5, TType.LIST, 'rootClusterIndexList', (TType.I32, None, False), None, ),  # 5
)
fix_spec(all_structs)
del all_structs



from __future__ import unicode_literals


def lun(lst):
    '''
    If lst is None return an empty list, else return lst.
    Simplifies iteration over list fields that might be unset.

    Args:
        lst (list): input list (or None)

    Return
        lst, or an empty list if lst is None
    '''
    return list() if (lst is None) else lst


def dun(d):
    '''
    If l is None return an empty dict, else return l.
    Simplifies iteration over dict fields that might be unset.

    Args:
        d (dict): input dict (or None)

    Return
        d, or an empty dict if d is None
    '''
    return dict() if (d is None) else d


def sun(s):
    '''
    If l is None return an empty set, else return l.
    Simplifies iteration over set fields that might be unset.

    Args:
        s (set): input set (or None)

    Return
        s, or an empty set if s is None
    '''
    return set() if (s is None) else s
